import AnyPlayer from './player/any-player';
import allHeroesJson from '../../../server/src/client-server-shared/all-heroes.json';

const state = {
  gameState: '',
  allHeroes: [],
  deckMode: '',
  advRules: [],
  'waitingFor/blue': false,
  'waitingFor/red': false,
  currentPlayer: '',
  'battleTiles/left': [],
  'battleTiles/center': [],
  'battleTiles/right': [],
  totalFood: 0,
  allHeroesJson
};

const getters = {
  state(state) {
    return state.gameState;
  },
  allHeroes(state) {
    return state.allHeroes;
  },
  heroById: state => id => {
    return state.allHeroes.find(hero => hero.id === id);
  },
  filterHeroes: state => filter => {
    console.log('filter :>> ', filter);
    // filter has
    //  - faction (orcs humans elves meca none all)
    //  - popularity (with without)
    //  - draft (yes no all)
    //  -  minPower <= power <= maxPower
    //  -  minCost <= cost <= maxCost
    //  - byName : if true filter name with filter.name
    //  - byId : [] of ids, deactivated if null or empty.
    if (!filter.faction) filter.faction = 'all';
    if (!filter.popularity) filter.popularity = 'without';
    if (!filter.minPower) filter.minPower = 0;
    if (!filter.maxPower) filter.maxPower = 0;
    if (!filter.minCost) filter.minCost = 0;
    if (!filter.maxCost) filter.minCost = 0;

    let draftFilter = 'all';
    switch (filter.draft) {
      case 'yes':
        draftFilter = true;
        break;
      case 'no':
        draftFilter = false;
        break;

      default:
        draftFilter = 'all';
        break;
    }
    console.log('state.allHeroes :>> ', state.allHeroes);
    return state.allHeroes.filter(hero => {
      if (filter.faction !== 'all' && hero.faction !== filter.faction)
        return false;
      if (hero.popularity !== 'any' && hero.popularity !== filter.popularity)
        return false;

      if (draftFilter !== 'all' && hero.isDraftable !== draftFilter)
        return false;

      if (hero.power < filter.minPower || hero.power > filter.maxPower)
        return false;
      if (hero.cost < filter.minCost || hero.cost > filter.maxCost)
        return false;

      if (
        filter.byName === true &&
        !hero.name.toLowerCase().includes(filter.name.toLowerCase())
      )
        return false;

      if (
        filter.byId &&
        filter.byId.length > 0 &&
        filter.byId.includes(hero.id)
      ) {
        console.log('not returning hero :>> ', hero);
        console.log('filter.byId :>> ', filter.byId);
        return false;
      }
      console.log('returning hero :>> ', hero);
      return true;
    });
  }
};

const mutations = {
  SET_HEROES: (state, payload) => {
    state.allHeroes = payload;
  },
  SET_FULL_GAME_STATE: (state, payload) => {
    Object.assign(state, payload);
  }
};

const actions = {
  setHeroes: ({ commit }, payload) => {
    commit('SET_HEROES', payload);
  },
  update: ({ commit }, payload) => {
    commit('SET_FULL_GAME_STATE', payload.game);
    commit('bluePlayer/SET_FULL_PLAYER_STATE', payload.bluePlayer);
    commit('redPlayer/SET_FULL_PLAYER_STATE', payload.redPlayer);
  }
};
const bluePlayer = new AnyPlayer();
const redPlayer = new AnyPlayer();
export default {
  namespaced: true,
  state,
  getters,
  mutations,
  actions,
  modules: {
    bluePlayer: {
      namespaced: true,
      ...bluePlayer
    },
    redPlayer: {
      namespaced: true,
      ...redPlayer
    }
  }
};
